@@ -39,11 +39,11 @@ typedef enum {
     ND_PROTO_DHCP                   = 18,
     ND_PROTO_POSTGRES               = 19,
     ND_PROTO_MYSQL                  = 20,
-    ND_PROTO_FTPS_CONTROL           = 21,
-    ND_PROTO_DIRECT_DOWNLOAD_LINK   = 22,
+    ND_PROTO_FTPS                   = 21,
+    ND_PROTO_DEPR22                 = 22,   // Deprecated: Direct Download Link
     ND_PROTO_MAIL_POPS              = 23,
-    ND_PROTO_APPLEJUICE             = 24,
-    ND_PROTO_DIRECTCONNECT          = 25,
+    ND_PROTO_DEPR24                 = 24,   // Deprecated: AppleJuice
+    ND_PROTO_DEPR25                 = 25,   // Deprecated: DirectConnect
     ND_PROTO_DEPR26                 = 26,   // Deprecated: NTOP
     ND_PROTO_COAP                   = 27,
     ND_PROTO_VMWARE                 = 28,
@@ -51,8 +51,8 @@ typedef enum {
     ND_PROTO_DEPR30                 = 30,   // Deprecated: Facebook Zero
     ND_PROTO_UBNTAC2                = 31,
     ND_PROTO_KONTIKI                = 32,
-    ND_PROTO_OPENFT                 = 33,
-    ND_PROTO_FASTTRACK              = 34,
+    ND_PROTO_DEPR33                 = 33,   // Deprecated: OpenFT
+    ND_PROTO_DEPR34                 = 34,   // Deprecated: FastTrack
     ND_PROTO_GNUTELLA               = 35,
     ND_PROTO_DEPR36                 = 36,   // Deprecated: eDonkey
     ND_PROTO_BITTORRENT             = 37,
@@ -74,17 +74,17 @@ typedef enum {
     ND_PROTO_DEPR53                 = 53,   // Deprecated: PPLive
     ND_PROTO_PPSTREAM               = 54,
     ND_PROTO_ZATTOO                 = 55,
-    ND_PROTO_SHOUTCAST              = 56,
-    ND_PROTO_SOPCAST                = 57,
+    ND_PROTO_DEPR56                 = 56,   // Deprecated: Shoutcast
+    ND_PROTO_DEPR57                 = 57,   // Deprecated: Sopcast
     ND_PROTO_DEPR58                 = 58,   // Deprecated: TVANTS
     ND_PROTO_TVUPLAYER              = 59,
     ND_PROTO_DEPR60                 = 60,   // Deprecated: HTTP_Download
     ND_PROTO_QQLIVE                 = 61,
-    ND_PROTO_THUNDER                = 62,
+    ND_PROTO_DEPR62                 = 62,   // Deprecated: Thunder
     ND_PROTO_DEPR63                 = 63,   // Deprecated: SoulSeek
     ND_PROTO_DEPR64                 = 64,   // Deprecated: SSL No Cert
     ND_PROTO_IRC                    = 65,
-    ND_PROTO_AYIYA                  = 66,
+    ND_PROTO_DEPR66                 = 66,   // Deprecated: Ayiya
     ND_PROTO_XMPP                   = 67,   // Renamed: Jabber
     ND_PROTO_FREE68                 = 68,
     ND_PROTO_FREE69                 = 69,
@@ -116,8 +116,8 @@ typedef enum {
     ND_PROTO_IAX                    = 95,
     ND_PROTO_TFTP                   = 96,
     ND_PROTO_AFP                    = 97,
-    ND_PROTO_STEALTHNET             = 98,
-    ND_PROTO_AIMINI                 = 99,
+    ND_PROTO_DEPR98                 = 98,   // Deprecated: StealthNet
+    ND_PROTO_DEPR99                 = 99,   // Deprecated: Aimini
     ND_PROTO_SIP                    = 100,
     ND_PROTO_TRUPHONE               = 101,
     ND_PROTO_IP_ICMPV6              = 102,
@@ -125,8 +125,8 @@ typedef enum {
     ND_PROTO_ARMAGETRON             = 104,
     ND_PROTO_CROSSFIRE              = 105,
     ND_PROTO_DOFUS                  = 106,
-    ND_PROTO_FIESTA                 = 107,
-    ND_PROTO_FLORENSIA              = 108,
+    ND_PROTO_DEPR107                = 107,  // Deprecated: Fiesta
+    ND_PROTO_DEPR108                = 108,  // Deprecated: Florensia
     ND_PROTO_GUILDWARS              = 109,
     ND_PROTO_FREE110                = 110,
     ND_PROTO_KERBEROS               = 111,
@@ -337,6 +337,23 @@ typedef enum {
     ND_PROTO_THREEMA                = 300,
     ND_PROTO_ALICLOUD               = 301,
     ND_PROTO_SYSLOGS                = 302,
+    ND_PROTO_NATPMP                 = 303,  // NAT Port Mapping Protocol
+    // TUYA LAN Protocol
+    // https://github.com/tuya/tuya-iotos-embeded-sdk-wifi-ble-bk7231n */
+    ND_PROTO_TUYA_LP                = 304,
+
+    ND_PROTO_ELASTICSEARCH          = 305,
+    ND_PROTO_AVAST                  = 306,
+    ND_PROTO_CRYNET                 = 307,
+    ND_PROTO_FASTCGI                = 308,
+    ND_PROTO_KISMET                 = 309,
+    ND_PROTO_LINE_CALL              = 310,
+    ND_PROTO_MUNIN                  = 311,
+    ND_PROTO_SYNCTHING              = 312,
+    ND_PROTO_TIVOCONNECT            = 313,
+    ND_PROTO_TPLINK_SHP             = 314,  // TP-LINK Smart Home Protocol
+    ND_PROTO_TAILSCALE              = 315,  // Tailscale
+    ND_PROTO_MERAKI_CLOUD           = 316,  // Meraki Cloud
 
     ND_PROTO_MAX,
     ND_PROTO_TODO                   = 0xffffffff
@@ -346,16 +363,14 @@ typedef unordered_map<unsigned, const char *> nd_protos_t;
 
 const nd_protos_t nd_protos = {
     { ND_PROTO_AFP, "AFP" },
-    { ND_PROTO_AIMINI, "Aimini" },
     { ND_PROTO_AJP, "AJP" },
     { ND_PROTO_ALICLOUD, "Alibaba/Cloud" },
     { ND_PROTO_AMONG_US, "AmongUs" },
     { ND_PROTO_AMQP, "AMQP" },
-    { ND_PROTO_APPLEJUICE, "APPLJ" },
     { ND_PROTO_APPLE_PUSH, "Apple/Push" },
     { ND_PROTO_ARMAGETRON, "Armagetron" },
+    { ND_PROTO_AVAST, "AVAST" },
     { ND_PROTO_AVAST_SDNS, "AVASTSecureDNS" },
-    { ND_PROTO_AYIYA, "Ayiya" },
     { ND_PROTO_BGP, "BGP" },
     { ND_PROTO_BITTORRENT, "BitTorrent" },
     { ND_PROTO_BJNP, "BJNP" },
@@ -371,15 +386,14 @@ const nd_protos_t nd_protos = {
     { ND_PROTO_CORBA, "Corba" },
     { ND_PROTO_CPHA, "CheckPointHA" },
     { ND_PROTO_CROSSFIRE, "Crossfire" },
+    { ND_PROTO_CRYNET, "CryNetwork" },
     { ND_PROTO_CSGO, "CSGO" },
     { ND_PROTO_DHCP, "DHCP" },
     { ND_PROTO_DHCPV6, "DHCPv6" },
     { ND_PROTO_DIAMETER, "Diameter" },
-    { ND_PROTO_DIRECTCONNECT, "DConnect" },
-    { ND_PROTO_DIRECT_DOWNLOAD_LINK, "DDLink" },
     { ND_PROTO_DNP3, "DNP3" },
-    { ND_PROTO_DNSCRYPT, "DNSCrypt" },
     { ND_PROTO_DNS, "DNS" },
+    { ND_PROTO_DNSCRYPT, "DNSCrypt" },
     { ND_PROTO_DOFUS, "Dofus" },
     { ND_PROTO_DOH, "DoH" },
     { ND_PROTO_DOQ, "DoQ" },
@@ -388,35 +402,35 @@ const nd_protos_t nd_protos = {
     { ND_PROTO_DROPBOX, "Dropbox" },
     { ND_PROTO_DTLS, "DTLS" },
     { ND_PROTO_EAQ, "EAQ" },
+    { ND_PROTO_ELASTICSEARCH, "ElasticSearch" },
     { ND_PROTO_ETHERNET_IP, "EtherNet/IP" },
-    { ND_PROTO_FASTTRACK, "FastTrack" },
-    { ND_PROTO_FIESTA, "Fiesta" },
+    { ND_PROTO_FASTCGI, "FastCGI" },
     { ND_PROTO_FIX, "FIX" },
-    { ND_PROTO_FLORENSIA, "Florensia" },
+    { ND_PROTO_FTPS, "FTP/S" },
     { ND_PROTO_FTP_CONTROL, "FTP/C" },
     { ND_PROTO_FTP_DATA, "FTP/D" },
-    { ND_PROTO_FTPS_CONTROL, "FTP/S/C" },
-    { ND_PROTO_FTPS_DATA, "FTP/S/D" },
     { ND_PROTO_GENSHIN_IMPACT, "Genshin/Impact" },
     { ND_PROTO_GIT, "Git" },
     { ND_PROTO_GNUTELLA, "Gnutella" },
     { ND_PROTO_GOOGLE_MEET_DUO, "Google/Meet/Duo" },
-    { ND_PROTO_GTP_C, "GTP/C" },
     { ND_PROTO_GTP, "GTP" },
+    { ND_PROTO_GTP_C, "GTP/C" },
     { ND_PROTO_GTP_P, "GTP/P" },
     { ND_PROTO_GTP_U, "GTP/U" },
     { ND_PROTO_GUILDWARS, "Guildwars" },
     { ND_PROTO_H323, "H323" },
     { ND_PROTO_HALFLIFE2, "HalfLife2" },
     { ND_PROTO_HP_VIRTGRP, "HP/VirtGrp" },
-    { ND_PROTO_HTTP_CONNECT, "HTTP/Connect" },
     { ND_PROTO_HTTP, "HTTP" },
-    { ND_PROTO_HTTP_PROXY, "HTTP/Proxy" },
     { ND_PROTO_HTTPS, "HTTP/S" },
+    { ND_PROTO_HTTP_CONNECT, "HTTP/Connect" },
+    { ND_PROTO_HTTP_PROXY, "HTTP/Proxy" },
     { ND_PROTO_I3D, "i3D" },
     { ND_PROTO_IAX, "IAX" },
     { ND_PROTO_ICECAST, "IceCast" },
     { ND_PROTO_IEC60870_5_104, "IEC60870/5/104" },
+    { ND_PROTO_IPP, "IPP" },
+    { ND_PROTO_IPSEC, "IPSEC" },
     { ND_PROTO_IP_EGP, "EGP" },
     { ND_PROTO_IP_GRE, "GRE" },
     { ND_PROTO_IP_ICMP, "ICMP" },
@@ -426,16 +440,16 @@ const nd_protos_t nd_protos = {
     { ND_PROTO_IP_OSPF, "OSPF" },
     { ND_PROTO_IP_PGM, "PGM" },
     { ND_PROTO_IP_PIM, "PIM" },
-    { ND_PROTO_IPP, "IPP" },
     { ND_PROTO_IP_SCTP, "SCTP" },
-    { ND_PROTO_IPSEC, "IPSEC" },
     { ND_PROTO_IP_VRRP, "VRRP" },
     { ND_PROTO_IRC, "IRC" },
     { ND_PROTO_IRCS, "IRC/S" },
     { ND_PROTO_KAKAOTALK_VOICE, "KakaoTalk/Voice" },
     { ND_PROTO_KERBEROS, "Kerberos" },
+    { ND_PROTO_KISMET, "KISMET" },
     { ND_PROTO_KONTIKI, "Kontiki" },
     { ND_PROTO_LDAP, "LDAP" },
+    { ND_PROTO_LINE_CALL, "Line/Call" },
     { ND_PROTO_LISP, "LISP" },
     { ND_PROTO_LLMNR, "LLMNR" },
     { ND_PROTO_LOTUS_NOTES, "LotusNotes" },
@@ -449,6 +463,7 @@ const nd_protos_t nd_protos = {
     { ND_PROTO_MDNS, "MDNS" },
     { ND_PROTO_MEGACO, "Megaco" },
     { ND_PROTO_MEMCACHED, "Memcached" },
+    { ND_PROTO_MERAKI_CLOUD, "Meraki/Cloud" },
     { ND_PROTO_MGCP, "MGCP" },
     { ND_PROTO_MINING, "Mining" },
     { ND_PROTO_MODBUS, "Modbus" },
@@ -458,7 +473,9 @@ const nd_protos_t nd_protos = {
     { ND_PROTO_MQTT, "MQTT" },
     { ND_PROTO_MQTTS, "MQTT/S" },
     { ND_PROTO_MSSQL_TDS, "MSSQL/TDS" },
+    { ND_PROTO_MUNIN, "Munin" },
     { ND_PROTO_MYSQL, "MYSQL" },
+    { ND_PROTO_NATPMP, "NAT/PMP" },
     { ND_PROTO_NATS, "NATS" },
     { ND_PROTO_NEST_LOG_SINK, "NestLog" },
     { ND_PROTO_NETBIOS, "NETBIOS" },
@@ -470,14 +487,13 @@ const nd_protos_t nd_protos = {
     { ND_PROTO_NOE, "NOE" },
     { ND_PROTO_NTP, "NTP" },
     { ND_PROTO_OOKLA, "OOKLA" },
-    { ND_PROTO_OPENFT, "OpenFT" },
     { ND_PROTO_OPENVPN, "OpenVPN" },
     { ND_PROTO_ORACLE, "Oracle" },
     { ND_PROTO_POSTGRES, "PGSQL" },
     { ND_PROTO_PPSTREAM, "PPStream" },
     { ND_PROTO_PPTP, "PPTP" },
-    { ND_PROTO_QQLIVE, "QQLive" },
     { ND_PROTO_QQ, "QQ" },
+    { ND_PROTO_QQLIVE, "QQLive" },
     { ND_PROTO_QUIC, "QUIC" },
     { ND_PROTO_RADIUS, "RADIUS" },
     { ND_PROTO_RAKNET, "RakNet" },
@@ -497,12 +513,11 @@ const nd_protos_t nd_protos = {
     { ND_PROTO_SAP, "SAP" },
     { ND_PROTO_SD_RTN, "SD/RTN" },
     { ND_PROTO_SFLOW, "SFlow" },
-    { ND_PROTO_SHOUTCAST, "ShoutCast" },
     { ND_PROTO_SIGNAL_CALL, "SignalCall" },
     { ND_PROTO_SIP, "SIP" },
     { ND_PROTO_SIPS, "SIP/S" },
-    { ND_PROTO_SKYPE_TEAMS_CALL, "Skype/Teams/Call" },
     { ND_PROTO_SKYPE_TEAMS, "Skype/Teams" },
+    { ND_PROTO_SKYPE_TEAMS_CALL, "Skype/Teams/Call" },
     { ND_PROTO_SMBV1, "SMBv1" },
     { ND_PROTO_SMBV23, "SMBv23" },
     { ND_PROTO_SMPP, "SMPP" },
@@ -511,16 +526,16 @@ const nd_protos_t nd_protos = {
     { ND_PROTO_SOAP, "SOAP" },
     { ND_PROTO_SOCKS, "SOCKS" },
     { ND_PROTO_SOMEIP, "SOMEIP" },
-    { ND_PROTO_SOPCAST, "Sopcast" },
     { ND_PROTO_SPOTIFY, "Spotify" },
     { ND_PROTO_SSDP, "SSDP" },
     { ND_PROTO_SSH, "SSH" },
     { ND_PROTO_STARCRAFT, "Starcraft" },
-    { ND_PROTO_STEALTHNET, "Stealthnet" },
     { ND_PROTO_STEAM, "Steam" },
     { ND_PROTO_STUN, "STUN" },
+    { ND_PROTO_SYNCTHING, "Syncthing" },
     { ND_PROTO_SYSLOG, "SYSLOG" },
     { ND_PROTO_SYSLOGS, "SYSLOG/S" },
+    { ND_PROTO_TAILSCALE, "Tailscale" },
     { ND_PROTO_TARGUS_GETDATA, "Targus/Dataspeed" },
     { ND_PROTO_TEAMSPEAK, "TeamSpeak" },
     { ND_PROTO_TEAMVIEWER, "TeamViewer" },
@@ -529,12 +544,14 @@ const nd_protos_t nd_protos = {
     { ND_PROTO_TEREDO, "Teredo" },
     { ND_PROTO_TFTP, "TFTP" },
     { ND_PROTO_THREEMA, "Threema" },
-    { ND_PROTO_THUNDER, "Thunder" },
     { ND_PROTO_TINC, "TINC" },
+    { ND_PROTO_TIVOCONNECT, "TiVo/Connect" },
     { ND_PROTO_TLS, "TLS" },
     { ND_PROTO_TOCA_BOCA, "TocaBoca" },
     { ND_PROTO_TODO, "TODO" },
+    { ND_PROTO_TPLINK_SHP, "TPLINK/SHP" },
     { ND_PROTO_TRUPHONE, "TruPhone" },
+    { ND_PROTO_TUYA_LP, "Tuya/LP" },
     { ND_PROTO_TVUPLAYER, "TVUplayer" },
     { ND_PROTO_UBNTAC2, "UBNTAC2" },
     { ND_PROTO_ULTRASURF, "UltraSurf" },
@@ -546,8 +563,8 @@ const nd_protos_t nd_protos = {
     { ND_PROTO_VXLAN, "VXLAN" },
     { ND_PROTO_WARCRAFT3, "Warcraft3" },
     { ND_PROTO_WEBSOCKET, "Websocket" },
-    { ND_PROTO_WHATSAPP_CALL, "WhatsApp/Call" },
     { ND_PROTO_WHATSAPP, "WhatsApp" },
+    { ND_PROTO_WHATSAPP_CALL, "WhatsApp/Call" },
     { ND_PROTO_WHOIS_DAS, "Whois/DAS" },
     { ND_PROTO_WIREGUARD, "WireGuard" },
     { ND_PROTO_WORLDOFKUNGFU, "WoKungFu" },
@@ -585,16 +602,14 @@ typedef unordered_map<uint16_t, nd_proto_id_t> nd_ndpi_proto_t;
 
 const nd_ndpi_proto_t nd_ndpi_protos = {
     { NDPI_PROTOCOL_AFP, ND_PROTO_AFP },
-    { NDPI_PROTOCOL_AIMINI, ND_PROTO_AIMINI },
     { NDPI_PROTOCOL_AJP, ND_PROTO_AJP },
     { NDPI_PROTOCOL_ALICLOUD, ND_PROTO_ALICLOUD },
     { NDPI_PROTOCOL_AMONG_US, ND_PROTO_AMONG_US },
     { NDPI_PROTOCOL_AMQP, ND_PROTO_AMQP },
-    { NDPI_PROTOCOL_APPLEJUICE, ND_PROTO_APPLEJUICE },
     { NDPI_PROTOCOL_APPLE_PUSH, ND_PROTO_APPLE_PUSH },
     { NDPI_PROTOCOL_ARMAGETRON, ND_PROTO_ARMAGETRON },
+    { NDPI_PROTOCOL_AVAST, ND_PROTO_AVAST },
     { NDPI_PROTOCOL_AVAST_SECUREDNS, ND_PROTO_AVAST_SDNS },
-    { NDPI_PROTOCOL_AYIYA, ND_PROTO_AYIYA },
     { NDPI_PROTOCOL_BGP, ND_PROTO_BGP },
     { NDPI_PROTOCOL_BITTORRENT, ND_PROTO_BITTORRENT },
     { NDPI_PROTOCOL_BJNP, ND_PROTO_BJNP },
@@ -608,34 +623,33 @@ const nd_ndpi_proto_t nd_ndpi_protos = {
     { NDPI_PROTOCOL_CORBA, ND_PROTO_CORBA },
     { NDPI_PROTOCOL_CPHA, ND_PROTO_CPHA },
     { NDPI_PROTOCOL_CROSSFIRE, ND_PROTO_CROSSFIRE },
+    { NDPI_PROTOCOL_CRYNET, ND_PROTO_CRYNET },
     { NDPI_PROTOCOL_CSGO, ND_PROTO_CSGO },
     { NDPI_PROTOCOL_DATASAVER, ND_PROTO_DATASAVER },
     { NDPI_PROTOCOL_DHCP, ND_PROTO_DHCP },
     { NDPI_PROTOCOL_DHCPV6, ND_PROTO_DHCPV6 },
     { NDPI_PROTOCOL_DIAMETER, ND_PROTO_DIAMETER },
-    { NDPI_PROTOCOL_DIRECTCONNECT, ND_PROTO_DIRECTCONNECT },
-    { NDPI_PROTOCOL_DIRECT_DOWNLOAD_LINK, ND_PROTO_DIRECT_DOWNLOAD_LINK },
     { NDPI_PROTOCOL_DNP3, ND_PROTO_DNP3 },
-    { NDPI_PROTOCOL_DNSCRYPT, ND_PROTO_DNSCRYPT },
     { NDPI_PROTOCOL_DNS, ND_PROTO_DNS },
+    { NDPI_PROTOCOL_DNSCRYPT, ND_PROTO_DNSCRYPT },
     { NDPI_PROTOCOL_DOFUS, ND_PROTO_DOFUS },
     { NDPI_PROTOCOL_DOH_DOT, ND_PROTO_DOQ },
     { NDPI_PROTOCOL_DRDA, ND_PROTO_DRDA },
     { NDPI_PROTOCOL_DROPBOX, ND_PROTO_DROPBOX },
     { NDPI_PROTOCOL_DTLS, ND_PROTO_DTLS },
     { NDPI_PROTOCOL_EAQ, ND_PROTO_EAQ },
+    { NDPI_PROTOCOL_ELASTICSEARCH, ND_PROTO_ELASTICSEARCH },
     { NDPI_PROTOCOL_ETHERNET_IP, ND_PROTO_ETHERNET_IP },
-    { NDPI_PROTOCOL_FASTTRACK, ND_PROTO_FASTTRACK },
-    { NDPI_PROTOCOL_FIESTA, ND_PROTO_FIESTA },
+    { NDPI_PROTOCOL_FASTCGI, ND_PROTO_FASTCGI },
     { NDPI_PROTOCOL_FIX, ND_PROTO_FIX },
-    { NDPI_PROTOCOL_FLORENSIA, ND_PROTO_FLORENSIA },
+    { NDPI_PROTOCOL_FTPS, ND_PROTO_FTPS },
     { NDPI_PROTOCOL_FTP_CONTROL, ND_PROTO_FTP_CONTROL },
     { NDPI_PROTOCOL_FTP_DATA, ND_PROTO_FTP_DATA },
     { NDPI_PROTOCOL_GENSHIN_IMPACT, ND_PROTO_GENSHIN_IMPACT },
     { NDPI_PROTOCOL_GIT, ND_PROTO_GIT },
     { NDPI_PROTOCOL_GNUTELLA, ND_PROTO_GNUTELLA },
-    { NDPI_PROTOCOL_GTP_C, ND_PROTO_GTP_C },
     { NDPI_PROTOCOL_GTP, ND_PROTO_GTP },
+    { NDPI_PROTOCOL_GTP_C, ND_PROTO_GTP_C },
     { NDPI_PROTOCOL_GTP_PRIME, ND_PROTO_GTP_P },
     { NDPI_PROTOCOL_GTP_U, ND_PROTO_GTP_U },
     { NDPI_PROTOCOL_GUILDWARS, ND_PROTO_GUILDWARS },
@@ -644,13 +658,15 @@ const nd_ndpi_proto_t nd_ndpi_protos = {
     { NDPI_PROTOCOL_HANGOUT_DUO, ND_PROTO_GOOGLE_MEET_DUO },
     { NDPI_PROTOCOL_HPVIRTGRP, ND_PROTO_HP_VIRTGRP },
     { NDPI_PROTOCOL_HSRP, ND_PROTO_CISCO_HSRP },
-    { NDPI_PROTOCOL_HTTP_CONNECT, ND_PROTO_HTTP_CONNECT },
     { NDPI_PROTOCOL_HTTP, ND_PROTO_HTTP },
+    { NDPI_PROTOCOL_HTTP_CONNECT, ND_PROTO_HTTP_CONNECT },
     { NDPI_PROTOCOL_HTTP_PROXY, ND_PROTO_HTTP_PROXY },
     { NDPI_PROTOCOL_I3D, ND_PROTO_I3D },
     { NDPI_PROTOCOL_IAX, ND_PROTO_IAX },
     { NDPI_PROTOCOL_ICECAST, ND_PROTO_ICECAST },
     { NDPI_PROTOCOL_IEC60870, ND_PROTO_IEC60870_5_104 },
+    { NDPI_PROTOCOL_IPP, ND_PROTO_IPP },
+    { NDPI_PROTOCOL_IPSEC, ND_PROTO_IPSEC },
     { NDPI_PROTOCOL_IP_EGP, ND_PROTO_IP_EGP },
     { NDPI_PROTOCOL_IP_GRE, ND_PROTO_IP_GRE },
     { NDPI_PROTOCOL_IP_ICMP, ND_PROTO_IP_ICMP },
@@ -660,16 +676,16 @@ const nd_ndpi_proto_t nd_ndpi_protos = {
     { NDPI_PROTOCOL_IP_OSPF, ND_PROTO_IP_OSPF },
     { NDPI_PROTOCOL_IP_PGM, ND_PROTO_IP_PGM },
     { NDPI_PROTOCOL_IP_PIM, ND_PROTO_IP_PIM },
-    { NDPI_PROTOCOL_IPP, ND_PROTO_IPP },
     { NDPI_PROTOCOL_IP_SCTP, ND_PROTO_IP_SCTP },
-    { NDPI_PROTOCOL_IPSEC, ND_PROTO_IPSEC },
     { NDPI_PROTOCOL_IP_VRRP, ND_PROTO_IP_VRRP },
     { NDPI_PROTOCOL_IRC, ND_PROTO_IRC },
     { NDPI_PROTOCOL_JABBER, ND_PROTO_XMPP },
     { NDPI_PROTOCOL_KAKAOTALK_VOICE, ND_PROTO_KAKAOTALK_VOICE },
     { NDPI_PROTOCOL_KERBEROS, ND_PROTO_KERBEROS },
+    { NDPI_PROTOCOL_KISMET, ND_PROTO_KISMET },
     { NDPI_PROTOCOL_KONTIKI, ND_PROTO_KONTIKI },
     { NDPI_PROTOCOL_LDAP, ND_PROTO_LDAP },
+    { NDPI_PROTOCOL_LINE_CALL, ND_PROTO_LINE_CALL },
     { NDPI_PROTOCOL_LISP, ND_PROTO_LISP },
     { NDPI_PROTOCOL_LLMNR, ND_PROTO_LLMNR },
     { NDPI_PROTOCOL_LOTUS_NOTES, ND_PROTO_LOTUS_NOTES },
@@ -683,6 +699,7 @@ const nd_ndpi_proto_t nd_ndpi_protos = {
     { NDPI_PROTOCOL_MDNS, ND_PROTO_MDNS },
     { NDPI_PROTOCOL_MEGACO, ND_PROTO_MEGACO },
     { NDPI_PROTOCOL_MEMCACHED, ND_PROTO_MEMCACHED },
+    { NDPI_PROTOCOL_MERAKI_CLOUD, ND_PROTO_MERAKI_CLOUD },
     { NDPI_PROTOCOL_MGCP, ND_PROTO_MGCP },
     { NDPI_PROTOCOL_MINING, ND_PROTO_MINING },
     { NDPI_PROTOCOL_MODBUS, ND_PROTO_MODBUS },
@@ -691,7 +708,9 @@ const nd_ndpi_proto_t nd_ndpi_protos = {
     { NDPI_PROTOCOL_MPEGTS, ND_PROTO_MPEGTS },
     { NDPI_PROTOCOL_MQTT, ND_PROTO_MQTT },
     { NDPI_PROTOCOL_MSSQL_TDS, ND_PROTO_MSSQL_TDS },
+    { NDPI_PROTOCOL_MUNIN, ND_PROTO_MUNIN},
     { NDPI_PROTOCOL_MYSQL, ND_PROTO_MYSQL },
+    { NDPI_PROTOCOL_NATPMP, ND_PROTO_NATPMP },
     { NDPI_PROTOCOL_NATS, ND_PROTO_NATS },
     { NDPI_PROTOCOL_NEST_LOG_SINK, ND_PROTO_NEST_LOG_SINK },
     { NDPI_PROTOCOL_NETBIOS, ND_PROTO_NETBIOS },
@@ -701,7 +720,6 @@ const nd_ndpi_proto_t nd_ndpi_protos = {
     { NDPI_PROTOCOL_NOE, ND_PROTO_NOE },
     { NDPI_PROTOCOL_NTP, ND_PROTO_NTP },
     { NDPI_PROTOCOL_OOKLA, ND_PROTO_OOKLA },
-    { NDPI_PROTOCOL_OPENFT, ND_PROTO_OPENFT },
     { NDPI_PROTOCOL_OPENVPN, ND_PROTO_OPENVPN },
     { NDPI_PROTOCOL_ORACLE, ND_PROTO_ORACLE },
     { NDPI_PROTOCOL_POSTGRES, ND_PROTO_POSTGRES },
@@ -726,12 +744,11 @@ const nd_ndpi_proto_t nd_ndpi_protos = {
     { NDPI_PROTOCOL_SAP, ND_PROTO_SAP },
     { NDPI_PROTOCOL_SD_RTN, ND_PROTO_SD_RTN },
     { NDPI_PROTOCOL_SFLOW, ND_PROTO_SFLOW },
-    { NDPI_PROTOCOL_SHOUTCAST, ND_PROTO_SHOUTCAST },
     { NDPI_PROTOCOL_SIGNAL_VOIP, ND_PROTO_SIGNAL_CALL },
     { NDPI_PROTOCOL_SIP, ND_PROTO_SIP },
     { NDPI_PROTOCOL_SKINNY, ND_PROTO_CISCO_SKINNY },
-    { NDPI_PROTOCOL_SKYPE_TEAMS_CALL, ND_PROTO_SKYPE_TEAMS_CALL },
     { NDPI_PROTOCOL_SKYPE_TEAMS, ND_PROTO_SKYPE_TEAMS },
+    { NDPI_PROTOCOL_SKYPE_TEAMS_CALL, ND_PROTO_SKYPE_TEAMS_CALL },
     { NDPI_PROTOCOL_SMBV1, ND_PROTO_SMBV1 },
     { NDPI_PROTOCOL_SMBV23, ND_PROTO_SMBV23 },
     { NDPI_PROTOCOL_SMPP, ND_PROTO_SMPP },
@@ -740,15 +757,15 @@ const nd_ndpi_proto_t nd_ndpi_protos = {
     { NDPI_PROTOCOL_SOAP, ND_PROTO_SOAP },
     { NDPI_PROTOCOL_SOCKS, ND_PROTO_SOCKS },
     { NDPI_PROTOCOL_SOMEIP, ND_PROTO_SOMEIP },
-    { NDPI_PROTOCOL_SOPCAST, ND_PROTO_SOPCAST },
     { NDPI_PROTOCOL_SPOTIFY, ND_PROTO_SPOTIFY },
     { NDPI_PROTOCOL_SSDP, ND_PROTO_SSDP },
     { NDPI_PROTOCOL_SSH, ND_PROTO_SSH },
     { NDPI_PROTOCOL_STARCRAFT, ND_PROTO_STARCRAFT },
-    { NDPI_PROTOCOL_STEALTHNET, ND_PROTO_STEALTHNET },
     { NDPI_PROTOCOL_STEAM, ND_PROTO_STEAM },
     { NDPI_PROTOCOL_STUN, ND_PROTO_STUN },
+    { NDPI_PROTOCOL_SYNCTHING, ND_PROTO_SYNCTHING },
     { NDPI_PROTOCOL_SYSLOG, ND_PROTO_SYSLOG },
+    { NDPI_PROTOCOL_TAILSCALE, ND_PROTO_TAILSCALE },
     { NDPI_PROTOCOL_TARGUS_GETDATA, ND_PROTO_TARGUS_GETDATA },
     { NDPI_PROTOCOL_TEAMSPEAK, ND_PROTO_TEAMSPEAK },
     { NDPI_PROTOCOL_TEAMVIEWER, ND_PROTO_TEAMVIEWER },
@@ -757,12 +774,14 @@ const nd_ndpi_proto_t nd_ndpi_protos = {
     { NDPI_PROTOCOL_TEREDO, ND_PROTO_TEREDO },
     { NDPI_PROTOCOL_TFTP, ND_PROTO_TFTP },
     { NDPI_PROTOCOL_THREEMA, ND_PROTO_THREEMA },
-    { NDPI_PROTOCOL_THUNDER, ND_PROTO_THUNDER },
     { NDPI_PROTOCOL_TIKTOK, ND_PROTO_TIKTOK },
     { NDPI_PROTOCOL_TINC, ND_PROTO_TINC },
+    { NDPI_PROTOCOL_TIVOCONNECT, ND_PROTO_TIVOCONNECT },
     { NDPI_PROTOCOL_TLS, ND_PROTO_TLS },
     { NDPI_PROTOCOL_TOCA_BOCA, ND_PROTO_TOCA_BOCA },
+    { NDPI_PROTOCOL_TPLINK_SHP, ND_PROTO_TPLINK_SHP },
     { NDPI_PROTOCOL_TRUPHONE, ND_PROTO_TRUPHONE },
+    { NDPI_PROTOCOL_TUYA_LP, ND_PROTO_TUYA_LP },
     { NDPI_PROTOCOL_TVUPLAYER, ND_PROTO_TVUPLAYER },
     { NDPI_PROTOCOL_UBNTAC2, ND_PROTO_UBNTAC2 },
     { NDPI_PROTOCOL_ULTRASURF, ND_PROTO_ULTRASURF },
@@ -775,12 +794,12 @@ const nd_ndpi_proto_t nd_ndpi_protos = {
     { NDPI_PROTOCOL_VXLAN, ND_PROTO_VXLAN },
     { NDPI_PROTOCOL_WARCRAFT3, ND_PROTO_WARCRAFT3 },
     { NDPI_PROTOCOL_WEBSOCKET, ND_PROTO_WEBSOCKET },
-    { NDPI_PROTOCOL_WHATSAPP_CALL, ND_PROTO_WHATSAPP_CALL },
     { NDPI_PROTOCOL_WHATSAPP, ND_PROTO_WHATSAPP },
+    { NDPI_PROTOCOL_WHATSAPP_CALL, ND_PROTO_WHATSAPP_CALL },
     { NDPI_PROTOCOL_WHOIS_DAS, ND_PROTO_WHOIS_DAS },
     { NDPI_PROTOCOL_WIREGUARD, ND_PROTO_WIREGUARD },
-    { NDPI_PROTOCOL_WORLD_OF_KUNG_FU, ND_PROTO_WORLDOFKUNGFU },
     { NDPI_PROTOCOL_WORLDOFWARCRAFT, ND_PROTO_WORLDOFWARCRAFT },
+    { NDPI_PROTOCOL_WORLD_OF_KUNG_FU, ND_PROTO_WORLDOFKUNGFU },
     { NDPI_PROTOCOL_WSD, ND_PROTO_WSD },
     { NDPI_PROTOCOL_XBOX, ND_PROTO_XBOX },
     { NDPI_PROTOCOL_XDMCP, ND_PROTO_XDMCP },
@@ -792,9 +811,9 @@ const nd_ndpi_proto_t nd_ndpi_protos = {
     { NDPI_PROTOCOL_ZOOM, ND_PROTO_ZOOM },
 };
 
-typedef vector<uint16_t> nd_ndpi_disabled_t;
+typedef vector<uint16_t> nd_ndpi_disabled_protos_t;
 
-const nd_ndpi_disabled_t nd_ndpi_disabled = {
+const nd_ndpi_disabled_protos_t nd_ndpi_disabled_protos = {
     NDPI_PROTOCOL_1KXUN,        // Not a protocol (no dissector): ID# 295 (1kxun)
     NDPI_PROTOCOL_ACCUWEATHER,  // Not a protocol: ID# 280 (AccuWeather)
     NDPI_PROTOCOL_ACTIVISION,   // Not a protocol (no dissector): ID# 258 (Activision)
@@ -809,6 +828,7 @@ const nd_ndpi_disabled_t nd_ndpi_disabled = {
     NDPI_PROTOCOL_APPLE,        // Not a protocol: ID# 140 (Apple)
     NDPI_PROTOCOL_APPLE_SIRI,   // Not a protocol (no dissector): ID# 254 (AppleSiri)
     NDPI_PROTOCOL_APPLESTORE,   // Not a protocol: ID# 224 (AppleStore)
+    NDPI_PROTOCOL_APPLETVPLUS,  // Not a protocol (no dissector): ID# 317 (AppleTVPlus)
     NDPI_PROTOCOL_BADOO,        // Not a protocol: ID# 279 (Badoo)
     NDPI_PROTOCOL_BLOOMBERG,    // Not a protocol: ID# 246 (Bloomberg)
     NDPI_PROTOCOL_CACHEFLY,     // Not a protocol: ID# 289 (Cachefly)
@@ -817,8 +837,10 @@ const nd_ndpi_disabled_t nd_ndpi_disabled = {
     NDPI_PROTOCOL_CNN,          // Not a protocol: ID# 180 (CNN)
     NDPI_PROTOCOL_CRASHLYSTICS, // Not a protocol (no dissector): ID# 275 (Crashlytics)
     NDPI_PROTOCOL_CYBERSECURITY, // Not a protocol: ID# 283 (Cybersec)
+    NDPI_PROTOCOL_DAILYMOTION,  // Not a protocol: ID# 322 (Dailymotion)
     NDPI_PROTOCOL_DAZN,         // Not a protocol: ID# 292 (Dazn)
     NDPI_PROTOCOL_DEEZER,       // Not a protocol: ID# 210 (Deezer)
+    NDPI_PROTOCOL_DIRECTV,      // Not a protocol (no dissector): ID# 318 (DirecTV)
     NDPI_PROTOCOL_DISCORD,      // Not a protocol (no dissector): ID# 58 (Discord)
     NDPI_PROTOCOL_DISNEYPLUS,   // Not a protocol: ID# 71 (DisneyPlus)
     NDPI_PROTOCOL_EBAY,         // Not a protocol: ID# 179 (eBay)
@@ -840,16 +862,20 @@ const nd_ndpi_disabled_t nd_ndpi_disabled = {
     NDPI_PROTOCOL_GOOGLE_PLUS,  // Not a protocol: ID# 72 (GooglePlus)
     NDPI_PROTOCOL_GOOGLE_SERVICES, // Not a protocol: ID# 239 (GoogleServices)
     NDPI_PROTOCOL_GOTO,         // Not a protocol: ID# 293 (GoTo)
+    NDPI_PROTOCOL_HBO,          // Not a protocol: ID# 319 (HBO)
     NDPI_PROTOCOL_HOTSPOT_SHIELD, // Not a protocol: ID# 215 (HotspotShield)
     NDPI_PROTOCOL_HULU,         // Not a protocol: ID# 137 (Hulu)
     NDPI_PROTOCOL_ICLOUD_PRIVATE_RELAY, // Not a protocol (no dissector): ID# 277 (iCloudPrivateRelay)
     NDPI_PROTOCOL_IFLIX,        // Not a protocol: ID# 202 (IFLIX)
+    NDPI_PROTOCOL_IHEARTRADIO,  // Not a protocol: ID# 325 (IHeartRadio)
     NDPI_PROTOCOL_IMO,          // Weak, too many false-positives, and obscure/undocumented.
     NDPI_PROTOCOL_INSTAGRAM,    // Not a protocol: ID# 211 (Instagram)
     NDPI_PROTOCOL_KAKAOTALK,    // Not a protocol (see KAKAOTALK_VOICE): ID# 193 (KakaoTalk)
     NDPI_PROTOCOL_LASTFM,       // Not a protocol: ID# 134 (LastFM)
     NDPI_PROTOCOL_LIKEE,        // Not a protocol: ID# 261 (Likee)
+    NDPI_PROTOCOL_LINE,         // Not a protocol: ID# 315 (Line)
     NDPI_PROTOCOL_LINKEDIN,     // Not a protocol: ID# 233 (LinkedIn)
+    NDPI_PROTOCOL_LIVESTREAM,   // Not a protocol: ID# 323 (Livestream)
     NDPI_PROTOCOL_MESSENGER,    // Not a protocol (no dissector): ID# 157 (Messenger)
     NDPI_PROTOCOL_MICROSOFT_365, // Not a protocol: ID# 219 (Microsoft365)
     NDPI_PROTOCOL_MICROSOFT_AZURE, // Not a protocol (no dissector): ID# 276 (Azure)
@@ -871,22 +897,28 @@ const nd_ndpi_disabled_t nd_ndpi_disabled = {
     NDPI_PROTOCOL_PSIPHON,      // Not a protocol: ID# 303 (Psiphon)
     NDPI_PROTOCOL_REDDIT,       // Not a protocol: ID# 205 (Reddit)
     NDPI_PROTOCOL_SALESFORCE,   // Not a protocol: ID# 266 (Salesforce)
+    NDPI_PROTOCOL_SHOWTIME,     // Not a protocol: ID# 321 (Showtime)
     NDPI_PROTOCOL_SIGNAL,       // Not a protocol (see SIGNAL_VOIP): ID# 39 (Signal)
     NDPI_PROTOCOL_SINA,         // Not a protocol (no dissector): ID# 200 (Sina(Weibo))
+    NDPI_PROTOCOL_SIRIUSXMRADIO, // Not a protocol: ID# 328 (SiriusXMRadio)
     NDPI_PROTOCOL_SLACK,        // Not a protocol (no dissector): ID# 118 (Slack)
     NDPI_PROTOCOL_SNAPCHAT,     // Not a protocol (no dissector): ID# 199 (Snapchat)
     NDPI_PROTOCOL_SOFTETHER,    // Not a protocol (no dissector): ID# 290 (Softether)
     NDPI_PROTOCOL_SOUNDCLOUD,   // Not a protocol: ID# 234 (SoundCloud)
     NDPI_PROTOCOL_TENCENT,      // Not a protocol: ID# 285 (Tencent)
+    NDPI_PROTOCOL_TENCENTVIDEO, // Not a protocol: ID# 324 (Tencentvideo)
+    NDPI_PROTOCOL_TIDAL,        // Not a protocol: ID# 326 (Tidal)
     NDPI_PROTOCOL_TOR,          // Not a protocol (no dissector): ID# 163 (Tor)
     NDPI_PROTOCOL_TUENTI,       // Not a protocol: ID# 149 (Tuenti)
     NDPI_PROTOCOL_TUMBLR,       // Not a protocol: ID# 90 (Tumblr)
+    NDPI_PROTOCOL_TUNEIN,       // Not a protocol: ID# 327 (TuneIn)
     NDPI_PROTOCOL_TUNNELBEAR,   // Not a protocol (no dissector): ID# 299 (TunnelBear)
     NDPI_PROTOCOL_TWITCH,       // Not a protocol (no dissector): ID# 195 (Twitch)
     NDPI_PROTOCOL_TWITTER,      // Not a protocol: ID# 120 (Twitter)
     NDPI_PROTOCOL_UBUNTUONE,    // Not a protocol: ID# 169 (UbuntuONE)
     NDPI_PROTOCOL_VEVO,         // Not a protocol: ID# 186 (Vevo)
     NDPI_PROTOCOL_VIMEO,        // Not a protocol: ID# 267 (Vimeo)
+    NDPI_PROTOCOL_VUDU,         // Not a protocol: ID# 320 (Vudu)
     NDPI_PROTOCOL_WAZE,         // Not a protocol: ID# 135 (Waze)
     NDPI_PROTOCOL_WEBEX,        // Not a protocol (no dissector): ID# 141 (Webex)
     NDPI_PROTOCOL_WECHAT,       // Not a protocol (no dissector): ID# 197 (WeChat)
@@ -898,6 +930,22 @@ const nd_ndpi_disabled_t nd_ndpi_disabled = {
     NDPI_PROTOCOL_YOUTUBE_UPLOAD, // Not a protocol: ID# 136 (YouTubeUpload)
 };
 
+typedef vector<uint16_t> nd_ndpi_free_protos_t;
+
+const nd_ndpi_free_protos_t nd_ndpi_free_protos = {
+    NDPI_PROTOCOL_FREE_22,
+    NDPI_PROTOCOL_FREE_25,
+    NDPI_PROTOCOL_FREE_33,
+    NDPI_PROTOCOL_FREE_34,
+    NDPI_PROTOCOL_FREE_56,
+    NDPI_PROTOCOL_FREE_57,
+    NDPI_PROTOCOL_FREE_62,
+    NDPI_PROTOCOL_FREE_98,
+    NDPI_PROTOCOL_FREE_99,
+    NDPI_PROTOCOL_FREE_107,
+    NDPI_PROTOCOL_FREE_108,
+};
+
 typedef unordered_map<uint16_t, vector<pair<uint16_t, nd_proto_id_t>>> nd_ndpi_portmap_t;
 
 const nd_ndpi_portmap_t nd_ndpi_portmap = {
@@ -912,7 +960,7 @@ const nd_ndpi_portmap_t nd_ndpi_portmap = {
         { 993, ND_PROTO_MAIL_IMAPS },
         { 995, ND_PROTO_MAIL_POPS },
         { 989, ND_PROTO_FTPS_DATA },
-        { 990, ND_PROTO_FTPS_CONTROL },
+        { 990, ND_PROTO_FTPS },
         { 1883, ND_PROTO_MQTTS },
         { 5061, ND_PROTO_SIPS },
         { 6514, ND_PROTO_SYSLOGS },
