@@ -33,6 +33,7 @@
 #include <atomic>
 #include <regex>
 #include <mutex>
+#include <bitset>
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -55,6 +56,9 @@
 
 #include <netdb.h>
 #include <netinet/in.h>
+#include <net/if.h>
+#include <net/if_arp.h>
+#include <linux/if_packet.h>
 
 #define __FAVOR_BSD 1
 #include <netinet/tcp.h>
@@ -68,6 +72,8 @@
 #include <nlohmann/json.hpp>
 using json = nlohmann::json;
 
+#include <radix/radix_tree.hpp>
+
 #ifdef _ND_USE_CONNTRACK
 #include <libnetfilter_conntrack/libnetfilter_conntrack.h>
 #endif
@@ -91,13 +97,17 @@ using namespace std;
 #ifdef _ND_USE_INOTIFY
 #include "nd-inotify.h"
 #endif
+#include "nd-risks.h"
+#include "nd-serializer.h"
+#include "nd-packet.h"
+#include "nd-json.h"
+#include "nd-util.h"
+#include "nd-addr.h"
 #ifdef _ND_USE_NETLINK
 #include "nd-netlink.h"
 #endif
-#include "nd-json.h"
 #include "nd-apps.h"
 #include "nd-protos.h"
-#include "nd-risks.h"
 #include "nd-category.h"
 #include "nd-flow.h"
 #include "nd-flow-map.h"
@@ -116,7 +126,6 @@ using namespace std;
 #ifdef _ND_USE_PLUGINS
 #include "nd-plugin.h"
 #endif
-#include "nd-util.h"
 #include "nd-signal.h"
 #include "nd-napi.h"
 
@@ -139,6 +148,13 @@ ndGlobalConfig::ndGlobalConfig() :
     uuid_site(NULL),
     dhc_save(ndDHC_PERSISTENT),
     fhc_save(ndFHC_PERSISTENT),
+    capture_type(ndCT_NONE),
+    capture_read_timeout(ND_CAPTURE_READ_TIMEOUT),
+    tpv3_fanout_mode(ndFOM_DISABLED),
+    tpv3_fanout_flags(ndFOF_NONE),
+    tpv3_rb_block_size(ND_TPV3_RB_BLOCK_SIZE),
+    tpv3_rb_frame_size(ND_TPV3_RB_FRAME_SIZE),
+    tpv3_rb_blocks(ND_TPV3_RB_BLOCKS),
     h_flow(stderr),
     ca_capture_base(0),
     ca_conntrack(-1),
@@ -153,6 +169,7 @@ ndGlobalConfig::ndGlobalConfig() :
     digest_app_config{0},
     digest_legacy_config{0},
     fhc_purge_divisor(ND_FHC_PURGE_DIVISOR),
+    fm_buckets(ND_FLOW_MAP_BUCKETS),
     max_detection_pkts(ND_MAX_DETECTION_PKTS),
     max_fhc(ND_MAX_FHC_ENTRIES),
     max_flows(0),
@@ -267,7 +284,8 @@ int ndGlobalConfig::Load(const string &filename)
     if (netifyd_section.find("ssl_verify") != netifyd_section.end()) {
         ND_GF_SET_FLAG(ndGF_SSL_VERIFY,
             reader.GetBoolean("netifyd", "ssl_verify", true));
-    } else if (netifyd_section.find("ssl_verify_peer") != netifyd_section.end()) {
+    }
+    else if (netifyd_section.find("ssl_verify_peer") != netifyd_section.end()) {
         ND_GF_SET_FLAG(ndGF_SSL_VERIFY,
             reader.GetBoolean("netifyd", "ssl_verify_peer", true));
     }
@@ -303,6 +321,9 @@ int ndGlobalConfig::Load(const string &filename)
     ND_GF_SET_FLAG(ndGF_LOAD_DOMAINS,
         reader.GetBoolean("netifyd", "load_domains", true));
 
+    this->fm_buckets = (unsigned)reader.GetInteger(
+        "netifyd", "flow_map_buckets", ND_FLOW_MAP_BUCKETS);
+
     // Threading section
     this->ca_capture_base = (int16_t)reader.GetInteger(
         "threads", "capture_base", this->ca_capture_base);
@@ -317,6 +338,90 @@ int ndGlobalConfig::Load(const string &filename)
     this->ca_socket = (int16_t)reader.GetInteger(
         "threads", "socket", this->ca_socket);
 
+    // Capture type section
+    string capture_type = reader.Get(
+        "capture", "type", "auto"
+    );
+
+    if (capture_type == "auto") {
+#if defined(_ND_USE_LIBPCAP)
+        this->capture_type = ndCT_PCAP;
+#elif defined(_ND_USE_TPACKETV3)
+        this->capture_type = ndCT_TPV3;
+#else
+#error "No available capture types!"
+#endif
+    }
+#if defined(_ND_USE_LIBPCAP)
+    else if (capture_type == "pcap")
+        this->capture_type = ndCT_PCAP;
+#endif
+#if defined(_ND_USE_TPACKETV3)
+    else if (capture_type == "tpv3")
+        this->capture_type = ndCT_TPV3;
+#endif
+    else {
+        fprintf(stderr, "Invalid capture type: %s\n",
+            capture_type.c_str()
+        );
+        return -1;
+    }
+
+    this->capture_read_timeout = (unsigned)reader.GetInteger(
+        "capture", "read_timeout", ND_CAPTURE_READ_TIMEOUT);
+
+    string tpv3_fanout_mode = reader.Get(
+        "capture", "tpv3_fanout_mode", "disabled"
+    );
+
+    if (tpv3_fanout_mode == "hash")
+        this->tpv3_fanout_mode = ndFOM_HASH;
+    else if (tpv3_fanout_mode == "lb" ||
+            tpv3_fanout_mode == "load_balanced")
+        this->tpv3_fanout_mode = ndFOM_LOAD_BALANCED;
+    else if (tpv3_fanout_mode == "cpu")
+        this->tpv3_fanout_mode = ndFOM_CPU;
+    else if (tpv3_fanout_mode == "rollover")
+        this->tpv3_fanout_mode = ndFOM_ROLLOVER;
+    else if (tpv3_fanout_mode == "random")
+        this->tpv3_fanout_mode = ndFOM_RANDOM;
+    else
+        this->tpv3_fanout_mode = ndFOM_DISABLED;
+
+    string tpv3_fanout_flags = reader.Get(
+        "capture", "tpv3_fanout_flags", "none"
+    );
+
+    if (tpv3_fanout_flags != "none") {
+        stringstream ss(tpv3_fanout_flags);
+
+        while (ss.good()) {
+            string flag;
+            getline(ss, flag, '|');
+
+            nd_trim(flag, ' ');
+
+            if (flag == "defrag")
+                this->tpv3_fanout_flags |= ndFOF_DEFRAG;
+            else if (flag == "rollover")
+                this->tpv3_fanout_flags |= ndFOF_ROLLOVER;
+            else {
+                fprintf(stderr, "Invalid fanout flag: %s\n",
+                    flag.c_str()
+                );
+            }
+        }
+    }
+
+    tpv3_rb_block_size = (unsigned)reader.GetInteger(
+        "capture", "tpv3_rb_block_size", ND_TPV3_RB_BLOCK_SIZE);
+
+    tpv3_rb_frame_size = (unsigned)reader.GetInteger(
+        "capture", "tpv3_rb_frame_size", ND_TPV3_RB_FRAME_SIZE);
+
+    tpv3_rb_blocks = (unsigned)reader.GetInteger(
+        "capture", "tpv3_rb_blocks", ND_TPV3_RB_BLOCKS);
+
     // Flow Hash Cache section
     ND_GF_SET_FLAG(ndGF_USE_FHC,
         reader.GetBoolean("flow_hash_cache", "enable", true));
